SystemOrganization addCategory: #'SystemEditor-Tests'!
SystemOrganization addCategory: #'SystemEditor-Editors'!

TestCase subclass: #ClassEditorTest
	instanceVariableNames: 'editor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Tests'!

ClassEditorTest subclass: #ClassEditorOntologyTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Tests'!

!ClassEditorOntologyTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 15:06'!
test01Creation
	"This invariant allows editors and their subject classes to use
	exactly the same reflection protocol, so that the only modification
	required to allow tools to modify the system atomically is to send
	#commit when all modifications are complete."

	self assert: editor subject == EditorSubject.
	self assert: editor class subject == EditorSubject class! !

!ClassEditorOntologyTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 15:06'!
test02Printing
	"This makes debugging easier."
	
	self assert: editor printString = 'a ClassEditor on: EditorSubject'.
	self assert: editor class printString = 'a MetaclassEditor on: EditorSubject'.! !

!ClassEditorOntologyTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 15:07'!
test03ClassReferences

	"The instance is a strange beast. Its class is created on the fly, and is
	both an instance of MetaclassEditor and a subclass of ClassEditor. 
	See #testCreation." 
	self assert: (editor class class == MetaclassEditor).
	self assert: (editor class superclass == ClassEditor).


	"editor is an instance of a light-weight class, so we don't register it
	with its superclass. In fact, the only thing that keeps it from being
	garbage collected is the class pointer from editor."
	self deny: (ClassEditor subclasses includes: editor class).

! !

!ClassEditorOntologyTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 15:07'!
test04IsKindOfClass
	"A class and its editor are polymorphic, but this kind of
	explicit test can tell the difference."

	self deny: (editor class isKindOf: Class)! !

!ClassEditorOntologyTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 15:24'!
test05ClassPool
	"To have DoIts be executed in the right context by a debugger or inspector,
	the generated class needs to be adoptable by FakeClassPool."

	self 
		shouldnt: [FakeClassPool adopt: editor class]
		raise: Error.! !

ClassEditorTest subclass: #ClassEditorProtocolTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Tests'!

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 15:40'!
test01InstVarNames
	self assert: editor instVarNames = editor subject instVarNames! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 15:36'!
test02AddInstVarName
	editor addInstVarName: 'gamma'.
	self assert: editor instVarNames = (editor subject instVarNames copyWith: 'gamma')! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 15:58'!
test03RemoveClassVarName
	editor removeClassVarName: 'Beth'.
	self assert: editor classVarNames = (editor subject classVarNames copyWithout: 'Beth')! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 15:40'!
test04ClassVarNames
	self assert: editor classVarNames = editor subject classVarNames! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 15:56'!
test05AddClassVarName
	editor addClassVarName: 'Gimel'.
	self assert: editor classVarNames = (editor subject classVarNames copyWith: 'Gimel')! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 15:59'!
test06RemoveClassVarName
	editor removeClassVarName: 'Beth'.
	self assert: editor classVarNames = (editor subject classVarNames copyWithout: 'Beth')! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 16:00'!
test07SharedPools
	self assert: editor sharedPools = editor subject sharedPools! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 16:03'!
test08AddSharedPool
	| pool |
	pool _ Object new.
	editor addSharedPool: pool.
	self assert: editor sharedPools = (editor subject sharedPools copyWith: pool)! !

!ClassEditorProtocolTest methodsFor: 'tests' stamp: 'cwp 10/23/2005 16:38'!
test09RemoveSharedPool
	| pool |
	pool _ Dictionary new.
	[editor subject addSharedPool: pool.
	editor removeSharedPool: pool.
	self assert: editor sharedPools isEmpty.
	self deny: editor subject sharedPools isEmpty]
		ensure:	[editor subject removeSharedPool: pool]! !

!ClassEditorTest class methodsFor: 'testing' stamp: 'cwp 10/23/2005 15:31'!
isAbtract
	^ self name == #ClassEditorTest! !

!ClassEditorTest methodsFor: 'running' stamp: 'cwp 10/23/2005 15:29'!
setUp
	editor := ClassEditor on: EditorSubject! !

Object subclass: #ClassEditor
	instanceVariableNames: 'subject instVarNames classVarNames sharedPools'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Editors'!

!ClassEditor class methodsFor: 'instance creation' stamp: 'cwp 10/23/2005 14:35'!
on: aClass
	| meta instance |
	meta _ MetaclassEditor new.
	instance _ meta new.
	instance setSubject: aClass.
	^ instance! !

!ClassEditor methodsFor: 'editing' stamp: 'cwp 10/23/2005 15:46'!
addClassVarName: aString 
	classVarNames ifNil: [classVarNames _ subject classVarNames].
	classVarNames _ classVarNames copyWith: aString! !

!ClassEditor methodsFor: 'editing' stamp: 'cwp 10/23/2005 15:36'!
addInstVarName: aString 
	instVarNames ifNil: [instVarNames _ subject instVarNames].
	instVarNames _ instVarNames copyWith: aString! !

!ClassEditor methodsFor: 'editing' stamp: 'cwp 10/23/2005 16:03'!
addSharedPool: aSharedPool 
	sharedPools ifNil: [sharedPools _ subject sharedPools].
	sharedPools _ sharedPools copyWith: aSharedPool! !

!ClassEditor methodsFor: 'reflection' stamp: 'cwp 10/23/2005 15:41'!
classVarNames
	^ classVarNames ifNil: [subject classVarNames]! !

!ClassEditor methodsFor: 'reflection' stamp: 'cwp 10/23/2005 15:37'!
instVarNames
	^ instVarNames ifNil: [subject instVarNames]! !

!ClassEditor methodsFor: 'printing' stamp: 'cwp 10/23/2005 02:58'!
printOn: aStream
	aStream 
		nextPutAll: 'a ClassEditor on: ';
		nextPutAll: subject name! !

!ClassEditor methodsFor: 'editing' stamp: 'cwp 10/23/2005 15:58'!
removeClassVarName: aString 
	classVarNames ifNil: [classVarNames _ subject classVarNames].
	classVarNames _ classVarNames copyWithout: aString! !

!ClassEditor methodsFor: 'editing' stamp: 'cwp 10/23/2005 15:39'!
removeInstVarName: aString 
	instVarNames ifNil: [instVarNames _ subject instVarNames].
	instVarNames _ instVarNames copyWithout: aString! !

!ClassEditor methodsFor: 'editing' stamp: 'cwp 10/23/2005 16:06'!
removeSharedPool: aSharedPool 
	sharedPools ifNil: [sharedPools _ subject sharedPools].
	sharedPools _ sharedPools copyWithout: aSharedPool! !

!ClassEditor methodsFor: 'initialize-release' stamp: 'cwp 10/23/2005 15:36'!
setSubject: aClass
	subject _ aClass.
	! !

!ClassEditor methodsFor: 'reflection' stamp: 'cwp 10/23/2005 16:01'!
sharedPools
	^ sharedPools ifNil: [subject sharedPools]! !

!ClassEditor methodsFor: 'accessing' stamp: 'cwp 10/23/2005 01:54'!
subject
	^ subject! !

Object subclass: #EditorSubject
	instanceVariableNames: 'alpha beta'
	classVariableNames: 'Aleph Beth'
	poolDictionaries: ''
	category: 'SystemEditor-Tests'!

Behavior subclass: #MetaclassEditor
	instanceVariableNames: 'editor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SystemEditor-Editors'!

!MetaclassEditor class methodsFor: 'instance creation' stamp: 'cwp 10/23/2005 14:38'!
new
	| instance |
	instance _ self basicNew.
	instance
		superclass: ClassEditor
		methodDictionary: MethodDictionary new
		format: ClassEditor format.
	^ instance! !

!MetaclassEditor methodsFor: 'accessing' stamp: 'cwp 10/23/2005 15:18'!
classPool
	^ Dictionary new! !

!MetaclassEditor methodsFor: 'metaclass' stamp: 'cwp 10/23/2005 14:43'!
new
	^ editor _ self basicNew! !

!MetaclassEditor methodsFor: 'printing' stamp: 'cwp 10/23/2005 14:45'!
printOn: aStream
	aStream
		nextPutAll: 'a MetaclassEditor on: ';
		print: editor subject ! !

!MetaclassEditor methodsFor: 'accessing' stamp: 'cwp 10/23/2005 14:45'!
subject
	^ editor subject class! !
